@@ -22,6 +22,7 @@ sub new {
   my $self = {
     'main'	=> shift,
     'audit'	=> shift,
+    'is_spamassassin_wrapper_object'	=> 1
   };
   bless ($self, $class);
   $self;
@@ -29,6 +30,13 @@ sub new {
 
 ###########################################################################
 
+sub get_mail_audit {
+  my ($self) = @_;
+  return $self->{audit};
+}
+
+###########################################################################
+
 sub get_header {
   my ($self, $hdr) = @_;
   $self->{audit}->get ($hdr);
